@@ -1,19 +1,52 @@
-from mypy.backports import OrderedDict
-from typing import List, Optional, Tuple, Callable
+from __future__ import annotations
 
-from mypy.types import (
-    Type, AnyType, TypeVisitor, UnboundType, NoneType, TypeVarType, Instance, CallableType,
-    TupleType, TypedDictType, ErasedType, UnionType, PartialType, DeletedType,
-    UninhabitedType, TypeType, TypeOfAny, Overloaded, FunctionLike, LiteralType,
-    ProperType, get_proper_type, get_proper_types, TypeAliasType, TypeGuardedType,
-    ParamSpecType, Parameters, UnpackType,
-)
-from mypy.subtypes import is_equivalent, is_subtype, is_callable_compatible, is_proper_subtype
+from typing import Callable
+
+from mypy import join
 from mypy.erasetype import erase_type
 from mypy.maptype import map_instance_to_supertype
-from mypy.typeops import tuple_fallback, make_simplified_union, is_recursive_pair
 from mypy.state import state
-from mypy import join
+from mypy.subtypes import (
+    is_callable_compatible,
+    is_equivalent,
+    is_proper_subtype,
+    is_same_type,
+    is_subtype,
+)
+from mypy.typeops import is_recursive_pair, make_simplified_union, tuple_fallback
+from mypy.types import (
+    MYPYC_NATIVE_INT_NAMES,
+    AnyType,
+    CallableType,
+    DeletedType,
+    ErasedType,
+    FunctionLike,
+    Instance,
+    LiteralType,
+    NoneType,
+    Overloaded,
+    Parameters,
+    ParamSpecType,
+    PartialType,
+    ProperType,
+    TupleType,
+    Type,
+    TypeAliasType,
+    TypedDictType,
+    TypeGuardedType,
+    TypeOfAny,
+    TypeType,
+    TypeVarLikeType,
+    TypeVarTupleType,
+    TypeVarType,
+    TypeVisitor,
+    UnboundType,
+    UninhabitedType,
+    UnionType,
+    UnpackType,
+    get_proper_type,
+    get_proper_types,
+)
 
 # TODO Describe this module.
 
@@ -35,17 +68,41 @@ def meet_types(s: Type, t: Type) -> ProperType:
     """Return the greatest lower bound of two types."""
     if is_recursive_pair(s, t):
         # This case can trigger an infinite recursion, general support for this will be
-        # tricky so we use a trivial meet (like for protocols).
+        # tricky, so we use a trivial meet (like for protocols).
         return trivial_meet(s, t)
     s = get_proper_type(s)
     t = get_proper_type(t)
 
+    if isinstance(s, Instance) and isinstance(t, Instance) and s.type == t.type:
+        # Code in checker.py should merge any extra_items where possible, so we
+        # should have only compatible extra_items here. We check this before
+        # the below subtype check, so that extra_attrs will not get erased.
+        if (s.extra_attrs or t.extra_attrs) and is_same_type(s, t):
+            if s.extra_attrs and t.extra_attrs:
+                if len(s.extra_attrs.attrs) > len(t.extra_attrs.attrs):
+                    # Return the one that has more precise information.
+                    return s
+                return t
+            if s.extra_attrs:
+                return s
+            return t
+
+    if not isinstance(s, UnboundType) and not isinstance(t, UnboundType):
+        if is_proper_subtype(s, t, ignore_promotions=True):
+            return s
+        if is_proper_subtype(t, s, ignore_promotions=True):
+            return t
+
     if isinstance(s, ErasedType):
         return s
     if isinstance(s, AnyType):
         return t
     if isinstance(s, UnionType) and not isinstance(t, UnionType):
         s, t = t, s
+
+    # Meets/joins require callable type normalization.
+    s, t = join.normalize_callables(s, t)
+
     return t.accept(TypeMeetVisitor(s))
 
 
@@ -56,48 +113,79 @@ def narrow_declared_type(declared: Type, narrowed: Type) -> Type:
         # A type guard forces the new type even if it doesn't overlap the old.
         return narrowed.type_guard
 
+    original_declared = declared
+    original_narrowed = narrowed
     declared = get_proper_type(declared)
     narrowed = get_proper_type(narrowed)
 
     if declared == narrowed:
-        return declared
+        return original_declared
     if isinstance(declared, UnionType):
-        return make_simplified_union([narrow_declared_type(x, narrowed)
-                                      for x in declared.relevant_items()])
+        return make_simplified_union(
+            [
+                narrow_declared_type(x, narrowed)
+                for x in declared.relevant_items()
+                # This (ugly) special-casing is needed to support checking
+                # branches like this:
+                # x: Union[float, complex]
+                # if isinstance(x, int):
+                #     ...
+                if (
+                    is_overlapping_types(x, narrowed, ignore_promotions=True)
+                    or is_subtype(narrowed, x, ignore_promotions=False)
+                )
+            ]
+        )
     if is_enum_overlapping_union(declared, narrowed):
-        return narrowed
-    elif not is_overlapping_types(declared, narrowed,
-                                  prohibit_none_typevar_overlap=True):
+        return original_narrowed
+    elif not is_overlapping_types(declared, narrowed, prohibit_none_typevar_overlap=True):
         if state.strict_optional:
             return UninhabitedType()
         else:
             return NoneType()
     elif isinstance(narrowed, UnionType):
-        return make_simplified_union([narrow_declared_type(declared, x)
-                                      for x in narrowed.relevant_items()])
+        return make_simplified_union(
+            [narrow_declared_type(declared, x) for x in narrowed.relevant_items()]
+        )
     elif isinstance(narrowed, AnyType):
-        return narrowed
+        return original_narrowed
     elif isinstance(narrowed, TypeVarType) and is_subtype(narrowed.upper_bound, declared):
         return narrowed
     elif isinstance(declared, TypeType) and isinstance(narrowed, TypeType):
         return TypeType.make_normalized(narrow_declared_type(declared.item, narrowed.item))
-    elif (isinstance(declared, TypeType)
-          and isinstance(narrowed, Instance)
-          and narrowed.type.is_metaclass()):
+    elif (
+        isinstance(declared, TypeType)
+        and isinstance(narrowed, Instance)
+        and narrowed.type.is_metaclass()
+    ):
         # We'd need intersection types, so give up.
-        return declared
-    elif isinstance(declared, (Instance, TupleType, TypeType, LiteralType)):
-        return meet_types(declared, narrowed)
+        return original_declared
+    elif isinstance(declared, Instance):
+        if declared.type.alt_promote:
+            # Special case: low-level integer type can't be narrowed
+            return original_declared
+        if (
+            isinstance(narrowed, Instance)
+            and narrowed.type.alt_promote
+            and narrowed.type.alt_promote.type is declared.type
+        ):
+            # Special case: 'int' can't be narrowed down to a native int type such as
+            # i64, since they have different runtime representations.
+            return original_declared
+        return meet_types(original_declared, original_narrowed)
+    elif isinstance(declared, (TupleType, TypeType, LiteralType)):
+        return meet_types(original_declared, original_narrowed)
     elif isinstance(declared, TypedDictType) and isinstance(narrowed, Instance):
         # Special case useful for selecting TypedDicts from unions using isinstance(x, dict).
-        if (narrowed.type.fullname == 'builtins.dict' and
-                all(isinstance(t, AnyType) for t in get_proper_types(narrowed.args))):
-            return declared
-        return meet_types(declared, narrowed)
-    return narrowed
+        if narrowed.type.fullname == "builtins.dict" and all(
+            isinstance(t, AnyType) for t in get_proper_types(narrowed.args)
+        ):
+            return original_declared
+        return meet_types(original_declared, original_narrowed)
+    return original_narrowed
 
 
-def get_possible_variants(typ: Type) -> List[Type]:
+def get_possible_variants(typ: Type) -> list[Type]:
     """This function takes any "Union-like" type and returns a list of the available "options".
 
     Specifically, there are currently exactly three different types that can have
@@ -112,8 +200,8 @@ def get_possible_variants(typ: Type) -> List[Type]:
     If this function receives any other type, we return a list containing just that
     original type. (E.g. pretend the type was contained within a singleton union).
 
-    The only exception is regular TypeVars: we return a list containing that TypeVar's
-    upper bound.
+    The only current exceptions are regular TypeVars and ParamSpecs. For these "TypeVarLike"s,
+    we return a list containing that TypeVarLike's upper bound.
 
     This function is useful primarily when checking to see if two types are overlapping:
     the algorithm to check if two unions are overlapping is fundamentally the same as
@@ -129,6 +217,8 @@ def get_possible_variants(typ: Type) -> List[Type]:
             return typ.values
         else:
             return [typ.upper_bound]
+    elif isinstance(typ, ParamSpecType):
+        return [typ.upper_bound]
     elif isinstance(typ, UnionType):
         return list(typ.items)
     elif isinstance(typ, Overloaded):
@@ -142,32 +232,40 @@ def get_possible_variants(typ: Type) -> List[Type]:
 def is_enum_overlapping_union(x: ProperType, y: ProperType) -> bool:
     """Return True if x is an Enum, and y is an Union with at least one Literal from x"""
     return (
-        isinstance(x, Instance) and x.type.is_enum and
-        isinstance(y, UnionType) and
-        any(isinstance(p, LiteralType) and x.type == p.fallback.type
-            for p in (get_proper_type(z) for z in y.relevant_items()))
+        isinstance(x, Instance)
+        and x.type.is_enum
+        and isinstance(y, UnionType)
+        and any(
+            isinstance(p, LiteralType) and x.type == p.fallback.type
+            for p in (get_proper_type(z) for z in y.relevant_items())
+        )
     )
 
 
 def is_literal_in_union(x: ProperType, y: ProperType) -> bool:
     """Return True if x is a Literal and y is an Union that includes x"""
-    return (isinstance(x, LiteralType) and isinstance(y, UnionType) and
-            any(x == get_proper_type(z) for z in y.items))
+    return (
+        isinstance(x, LiteralType)
+        and isinstance(y, UnionType)
+        and any(x == get_proper_type(z) for z in y.items)
+    )
 
 
-def is_overlapping_types(left: Type,
-                         right: Type,
-                         ignore_promotions: bool = False,
-                         prohibit_none_typevar_overlap: bool = False) -> bool:
+def is_overlapping_types(
+    left: Type,
+    right: Type,
+    ignore_promotions: bool = False,
+    prohibit_none_typevar_overlap: bool = False,
+    ignore_uninhabited: bool = False,
+) -> bool:
     """Can a value of type 'left' also be of type 'right' or vice-versa?
 
     If 'ignore_promotions' is True, we ignore promotions while checking for overlaps.
     If 'prohibit_none_typevar_overlap' is True, we disallow None from overlapping with
     TypeVars (in both strict-optional and non-strict-optional mode).
     """
-    if (
-        isinstance(left, TypeGuardedType)  # type: ignore[misc]
-        or isinstance(right, TypeGuardedType)  # type: ignore[misc]
+    if isinstance(left, TypeGuardedType) or isinstance(  # type: ignore[misc]
+        right, TypeGuardedType
     ):
         # A type guard forces the new type even if it doesn't overlap the old.
         return True
@@ -175,13 +273,16 @@ def is_overlapping_types(left: Type,
     left, right = get_proper_types((left, right))
 
     def _is_overlapping_types(left: Type, right: Type) -> bool:
-        '''Encode the kind of overlapping check to perform.
+        """Encode the kind of overlapping check to perform.
 
-        This function mostly exists so we don't have to repeat keyword arguments everywhere.'''
+        This function mostly exists so we don't have to repeat keyword arguments everywhere."""
         return is_overlapping_types(
-            left, right,
+            left,
+            right,
             ignore_promotions=ignore_promotions,
-            prohibit_none_typevar_overlap=prohibit_none_typevar_overlap)
+            prohibit_none_typevar_overlap=prohibit_none_typevar_overlap,
+            ignore_uninhabited=ignore_uninhabited,
+        )
 
     # We should never encounter this type.
     if isinstance(left, PartialType) or isinstance(right, PartialType):
@@ -228,8 +329,11 @@ def is_overlapping_types(left: Type,
     ):
         return True
 
-    if (is_proper_subtype(left, right, ignore_promotions=ignore_promotions)
-            or is_proper_subtype(right, left, ignore_promotions=ignore_promotions)):
+    if is_proper_subtype(
+        left, right, ignore_promotions=ignore_promotions, ignore_uninhabited=ignore_uninhabited
+    ) or is_proper_subtype(
+        right, left, ignore_promotions=ignore_promotions, ignore_uninhabited=ignore_uninhabited
+    ):
         return True
 
     # See the docstring for 'get_possible_variants' for more info on what the
@@ -238,37 +342,56 @@ def is_overlapping_types(left: Type,
     left_possible = get_possible_variants(left)
     right_possible = get_possible_variants(right)
 
-    # We start by checking multi-variant types like Unions first. We also perform
-    # the same logic if either type happens to be a TypeVar.
+    # First handle special cases relating to PEP 612:
+    # - comparing a `Parameters` to a `Parameters`
+    # - comparing a `Parameters` to a `ParamSpecType`
+    # - comparing a `ParamSpecType` to a `ParamSpecType`
     #
-    # Handling the TypeVars now lets us simulate having them bind to the corresponding
+    # These should all always be considered overlapping equality checks.
+    # These need to be done before we move on to other TypeVarLike comparisons.
+    if isinstance(left, (Parameters, ParamSpecType)) and isinstance(
+        right, (Parameters, ParamSpecType)
+    ):
+        return True
+    # A `Parameters` does not overlap with anything else, however
+    if isinstance(left, Parameters) or isinstance(right, Parameters):
+        return False
+
+    # Now move on to checking multi-variant types like Unions. We also perform
+    # the same logic if either type happens to be a TypeVar/ParamSpec/TypeVarTuple.
+    #
+    # Handling the TypeVarLikes now lets us simulate having them bind to the corresponding
     # type -- if we deferred these checks, the "return-early" logic of the other
     # checks will prevent us from detecting certain overlaps.
     #
-    # If both types are singleton variants (and are not TypeVars), we've hit the base case:
+    # If both types are singleton variants (and are not TypeVarLikes), we've hit the base case:
     # we skip these checks to avoid infinitely recursing.
 
-    def is_none_typevar_overlap(t1: Type, t2: Type) -> bool:
+    def is_none_typevarlike_overlap(t1: Type, t2: Type) -> bool:
         t1, t2 = get_proper_types((t1, t2))
-        return isinstance(t1, NoneType) and isinstance(t2, TypeVarType)
+        return isinstance(t1, NoneType) and isinstance(t2, TypeVarLikeType)
 
     if prohibit_none_typevar_overlap:
-        if is_none_typevar_overlap(left, right) or is_none_typevar_overlap(right, left):
+        if is_none_typevarlike_overlap(left, right) or is_none_typevarlike_overlap(right, left):
             return False
 
-    if (len(left_possible) > 1 or len(right_possible) > 1
-            or isinstance(left, TypeVarType) or isinstance(right, TypeVarType)):
+    if (
+        len(left_possible) > 1
+        or len(right_possible) > 1
+        or isinstance(left, TypeVarLikeType)
+        or isinstance(right, TypeVarLikeType)
+    ):
         for l in left_possible:
             for r in right_possible:
                 if _is_overlapping_types(l, r):
                     return True
         return False
 
-    # Now that we've finished handling TypeVars, we're free to end early
+    # Now that we've finished handling TypeVarLikes, we're free to end early
     # if one one of the types is None and we're running in strict-optional mode.
     # (None only overlaps with None in strict-optional mode).
     #
-    # We must perform this check after the TypeVar checks because
+    # We must perform this check after the TypeVarLike checks because
     # a TypeVar could be bound to None, for example.
 
     if state.strict_optional and isinstance(left, NoneType) != isinstance(right, NoneType):
@@ -286,8 +409,7 @@ def is_overlapping_types(left: Type,
         return are_typed_dicts_overlapping(left, right, ignore_promotions=ignore_promotions)
     elif typed_dict_mapping_pair(left, right):
         # Overlaps between TypedDicts and Mappings require dedicated logic.
-        return typed_dict_mapping_overlap(left, right,
-                                          overlapping=_is_overlapping_types)
+        return typed_dict_mapping_overlap(left, right, overlapping=_is_overlapping_types)
     elif isinstance(left, TypedDictType):
         left = left.fallback
     elif isinstance(right, TypedDictType):
@@ -312,8 +434,8 @@ def is_overlapping_types(left: Type,
         """Special cases for type object types overlaps."""
         # TODO: these checks are a bit in gray area, adjust if they cause problems.
         left, right = get_proper_types((left, right))
-        # 1. Type[C] vs Callable[..., C], where the latter is class object.
-        if isinstance(left, TypeType) and isinstance(right, CallableType) and right.is_type_obj():
+        # 1. Type[C] vs Callable[..., C] overlap even if the latter is not class object.
+        if isinstance(left, TypeType) and isinstance(right, CallableType):
             return _is_overlapping_types(left.item, right.ret_type)
         # 2. Type[C] vs Meta, where Meta is a metaclass for C.
         if isinstance(left, TypeType) and isinstance(right, Instance):
@@ -322,9 +444,9 @@ def is_overlapping_types(left: Type,
                 if left_meta is not None:
                     return _is_overlapping_types(left_meta, right)
                 # builtins.type (default metaclass) overlaps with all metaclasses
-                return right.type.has_base('builtins.type')
+                return right.type.has_base("builtins.type")
             elif isinstance(left.item, AnyType):
-                return right.type.has_base('builtins.type')
+                return right.type.has_base("builtins.type")
         # 3. Callable[..., C] vs Meta is considered below, when we switch to fallbacks.
         return False
 
@@ -332,10 +454,13 @@ def is_overlapping_types(left: Type,
         return _type_object_overlap(left, right) or _type_object_overlap(right, left)
 
     if isinstance(left, CallableType) and isinstance(right, CallableType):
-        return is_callable_compatible(left, right,
-                                      is_compat=_is_overlapping_types,
-                                      ignore_pos_arg_names=True,
-                                      allow_partial_overlap=True)
+        return is_callable_compatible(
+            left,
+            right,
+            is_compat=_is_overlapping_types,
+            ignore_pos_arg_names=True,
+            allow_partial_overlap=True,
+        )
     elif isinstance(left, CallableType):
         left = left.fallback
     elif isinstance(right, CallableType):
@@ -359,8 +484,14 @@ def is_overlapping_types(left: Type,
     if isinstance(left, Instance) and isinstance(right, Instance):
         # First we need to handle promotions and structural compatibility for instances
         # that came as fallbacks, so simply call is_subtype() to avoid code duplication.
-        if (is_subtype(left, right, ignore_promotions=ignore_promotions)
-                or is_subtype(right, left, ignore_promotions=ignore_promotions)):
+        if is_subtype(
+            left, right, ignore_promotions=ignore_promotions, ignore_uninhabited=ignore_uninhabited
+        ) or is_subtype(
+            right, left, ignore_promotions=ignore_promotions, ignore_uninhabited=ignore_uninhabited
+        ):
+            return True
+
+        if right.type.fullname == "builtins.int" and left.type.fullname in MYPYC_NATIVE_INT_NAMES:
             return True
 
         # Two unrelated types cannot be partially overlapping: they're disjoint.
@@ -386,8 +517,10 @@ def is_overlapping_types(left: Type,
             # Or, to use a more concrete example, List[Union[A, B]] and List[Union[B, C]]
             # would be considered partially overlapping since it's possible for both lists
             # to contain only instances of B at runtime.
-            if all(_is_overlapping_types(left_arg, right_arg)
-                   for left_arg, right_arg in zip(left.args, right.args)):
+            if all(
+                _is_overlapping_types(left_arg, right_arg)
+                for left_arg, right_arg in zip(left.args, right.args)
+            ):
                 return True
 
         return False
@@ -398,37 +531,49 @@ def is_overlapping_types(left: Type,
     # Note: it's unclear however, whether returning False is the right thing
     # to do when inferring reachability -- see  https://github.com/python/mypy/issues/5529
 
-    assert type(left) != type(right)
+    assert type(left) != type(right), f"{type(left)} vs {type(right)}"
     return False
 
 
-def is_overlapping_erased_types(left: Type, right: Type, *,
-                                ignore_promotions: bool = False) -> bool:
+def is_overlapping_erased_types(
+    left: Type, right: Type, *, ignore_promotions: bool = False
+) -> bool:
     """The same as 'is_overlapping_erased_types', except the types are erased first."""
-    return is_overlapping_types(erase_type(left), erase_type(right),
-                                ignore_promotions=ignore_promotions,
-                                prohibit_none_typevar_overlap=True)
+    return is_overlapping_types(
+        erase_type(left),
+        erase_type(right),
+        ignore_promotions=ignore_promotions,
+        prohibit_none_typevar_overlap=True,
+    )
 
 
-def are_typed_dicts_overlapping(left: TypedDictType, right: TypedDictType, *,
-                                ignore_promotions: bool = False,
-                                prohibit_none_typevar_overlap: bool = False) -> bool:
+def are_typed_dicts_overlapping(
+    left: TypedDictType,
+    right: TypedDictType,
+    *,
+    ignore_promotions: bool = False,
+    prohibit_none_typevar_overlap: bool = False,
+) -> bool:
     """Returns 'true' if left and right are overlapping TypeDictTypes."""
     # All required keys in left are present and overlapping with something in right
     for key in left.required_keys:
         if key not in right.items:
             return False
-        if not is_overlapping_types(left.items[key], right.items[key],
-                                    ignore_promotions=ignore_promotions,
-                                    prohibit_none_typevar_overlap=prohibit_none_typevar_overlap):
+        if not is_overlapping_types(
+            left.items[key],
+            right.items[key],
+            ignore_promotions=ignore_promotions,
+            prohibit_none_typevar_overlap=prohibit_none_typevar_overlap,
+        ):
             return False
 
     # Repeat check in the other direction
     for key in right.required_keys:
         if key not in left.items:
             return False
-        if not is_overlapping_types(left.items[key], right.items[key],
-                                    ignore_promotions=ignore_promotions):
+        if not is_overlapping_types(
+            left.items[key], right.items[key], ignore_promotions=ignore_promotions
+        ):
             return False
 
     # The presence of any additional optional keys does not affect whether the two
@@ -437,26 +582,35 @@ def are_typed_dicts_overlapping(left: TypedDictType, right: TypedDictType, *,
     return True
 
 
-def are_tuples_overlapping(left: Type, right: Type, *,
-                           ignore_promotions: bool = False,
-                           prohibit_none_typevar_overlap: bool = False) -> bool:
+def are_tuples_overlapping(
+    left: Type,
+    right: Type,
+    *,
+    ignore_promotions: bool = False,
+    prohibit_none_typevar_overlap: bool = False,
+) -> bool:
     """Returns true if left and right are overlapping tuples."""
     left, right = get_proper_types((left, right))
     left = adjust_tuple(left, right) or left
     right = adjust_tuple(right, left) or right
-    assert isinstance(left, TupleType), 'Type {} is not a tuple'.format(left)
-    assert isinstance(right, TupleType), 'Type {} is not a tuple'.format(right)
+    assert isinstance(left, TupleType), f"Type {left} is not a tuple"
+    assert isinstance(right, TupleType), f"Type {right} is not a tuple"
     if len(left.items) != len(right.items):
         return False
-    return all(is_overlapping_types(l, r,
-                                    ignore_promotions=ignore_promotions,
-                                    prohibit_none_typevar_overlap=prohibit_none_typevar_overlap)
-               for l, r in zip(left.items, right.items))
+    return all(
+        is_overlapping_types(
+            l,
+            r,
+            ignore_promotions=ignore_promotions,
+            prohibit_none_typevar_overlap=prohibit_none_typevar_overlap,
+        )
+        for l, r in zip(left.items, right.items)
+    )
 
 
-def adjust_tuple(left: ProperType, r: ProperType) -> Optional[TupleType]:
+def adjust_tuple(left: ProperType, r: ProperType) -> TupleType | None:
     """Find out if `left` is a Tuple[A, ...], and adjust its length to `right`"""
-    if isinstance(left, Instance) and left.type.fullname == 'builtins.tuple':
+    if isinstance(left, Instance) and left.type.fullname == "builtins.tuple":
         n = r.length() if isinstance(r, TupleType) else 1
         return TupleType([left.args[0]] * n, left)
     return None
@@ -464,8 +618,9 @@ def adjust_tuple(left: ProperType, r: ProperType) -> Optional[TupleType]:
 
 def is_tuple(typ: Type) -> bool:
     typ = get_proper_type(typ)
-    return (isinstance(typ, TupleType)
-            or (isinstance(typ, Instance) and typ.type.fullname == 'builtins.tuple'))
+    return isinstance(typ, TupleType) or (
+        isinstance(typ, Instance) and typ.type.fullname == "builtins.tuple"
+    )
 
 
 class TypeMeetVisitor(TypeVisitor[ProperType]):
@@ -488,19 +643,19 @@ class TypeMeetVisitor(TypeVisitor[ProperType]):
 
     def visit_union_type(self, t: UnionType) -> ProperType:
         if isinstance(self.s, UnionType):
-            meets: List[Type] = []
+            meets: list[Type] = []
             for x in t.items:
                 for y in self.s.items:
                     meets.append(meet_types(x, y))
         else:
-            meets = [meet_types(x, self.s)
-                     for x in t.items]
+            meets = [meet_types(x, self.s) for x in t.items]
         return make_simplified_union(meets)
 
     def visit_none_type(self, t: NoneType) -> ProperType:
         if state.strict_optional:
-            if isinstance(self.s, NoneType) or (isinstance(self.s, Instance) and
-                                               self.s.type.fullname == 'builtins.object'):
+            if isinstance(self.s, NoneType) or (
+                isinstance(self.s, Instance) and self.s.type.fullname == "builtins.object"
+            ):
                 return t
             else:
                 return UninhabitedType()
@@ -536,6 +691,12 @@ class TypeMeetVisitor(TypeVisitor[ProperType]):
         else:
             return self.default(self.s)
 
+    def visit_type_var_tuple(self, t: TypeVarTupleType) -> ProperType:
+        if self.s == t:
+            return self.s
+        else:
+            return self.default(self.s)
+
     def visit_unpack_type(self, t: UnpackType) -> ProperType:
         raise NotImplementedError
 
@@ -556,7 +717,7 @@ class TypeMeetVisitor(TypeVisitor[ProperType]):
                 if is_subtype(t, self.s) or is_subtype(self.s, t):
                     # Combine type arguments. We could have used join below
                     # equivalently.
-                    args: List[Type] = []
+                    args: list[Type] = []
                     # N.B: We use zip instead of indexing because the lengths might have
                     # mismatches during daemon reprocessing.
                     for ta, sia in zip(t.args, self.s.args):
@@ -568,6 +729,12 @@ class TypeMeetVisitor(TypeVisitor[ProperType]):
                     else:
                         return NoneType()
             else:
+                alt_promote = t.type.alt_promote
+                if alt_promote and alt_promote.type is self.s.type:
+                    return t
+                alt_promote = self.s.type.alt_promote
+                if alt_promote and alt_promote.type is t.type:
+                    return self.s
                 if is_subtype(t, self.s):
                     return t
                 elif is_subtype(self.s, t):
@@ -603,8 +770,10 @@ class TypeMeetVisitor(TypeVisitor[ProperType]):
             result = meet_similar_callables(t, self.s)
             # We set the from_type_type flag to suppress error when a collection of
             # concrete class objects gets inferred as their common abstract superclass.
-            if not ((t.is_type_obj() and t.type_object().is_abstract) or
-                    (self.s.is_type_obj() and self.s.type_object().is_abstract)):
+            if not (
+                (t.is_type_obj() and t.type_object().is_abstract)
+                or (self.s.is_type_obj() and self.s.type_object().is_abstract)
+            ):
                 result.from_type_type = True
             if isinstance(get_proper_type(result.ret_type), UninhabitedType):
                 # Return a plain None or <uninhabited> instead of a weird function.
@@ -643,14 +812,14 @@ class TypeMeetVisitor(TypeVisitor[ProperType]):
 
     def visit_tuple_type(self, t: TupleType) -> ProperType:
         if isinstance(self.s, TupleType) and self.s.length() == t.length():
-            items: List[Type] = []
+            items: list[Type] = []
             for i in range(t.length()):
                 items.append(self.meet(t.items[i], self.s.items[i]))
             # TODO: What if the fallbacks are different?
             return TupleType(items, tuple_fallback(t))
         elif isinstance(self.s, Instance):
             # meet(Tuple[t1, t2, <...>], Tuple[s, ...]) == Tuple[meet(t1, s), meet(t2, s), <...>].
-            if self.s.type.fullname == 'builtins.tuple' and self.s.args:
+            if self.s.type.fullname == "builtins.tuple" and self.s.args:
                 return t.copy_modified(items=[meet_types(it, self.s.args[0]) for it in t.items])
             elif is_proper_subtype(t, self.s):
                 # A named tuple that inherits from a normal class
@@ -659,19 +828,20 @@ class TypeMeetVisitor(TypeVisitor[ProperType]):
 
     def visit_typeddict_type(self, t: TypedDictType) -> ProperType:
         if isinstance(self.s, TypedDictType):
-            for (name, l, r) in self.s.zip(t):
-                if (not is_equivalent(l, r) or
-                        (name in t.required_keys) != (name in self.s.required_keys)):
+            for name, l, r in self.s.zip(t):
+                if not is_equivalent(l, r) or (name in t.required_keys) != (
+                    name in self.s.required_keys
+                ):
                     return self.default(self.s)
-            item_list: List[Tuple[str, Type]] = []
-            for (item_name, s_item_type, t_item_type) in self.s.zipall(t):
+            item_list: list[tuple[str, Type]] = []
+            for item_name, s_item_type, t_item_type in self.s.zipall(t):
                 if s_item_type is not None:
                     item_list.append((item_name, s_item_type))
                 else:
                     # at least one of s_item_type and t_item_type is not None
                     assert t_item_type is not None
                     item_list.append((item_name, t_item_type))
-            items = OrderedDict(item_list)
+            items = dict(item_list)
             fallback = self.s.create_anonymous_fallback()
             required_keys = t.required_keys | self.s.required_keys
             return TypedDictType(items, required_keys, fallback)
@@ -690,7 +860,7 @@ class TypeMeetVisitor(TypeVisitor[ProperType]):
 
     def visit_partial_type(self, t: PartialType) -> ProperType:
         # We can't determine the meet of partial types. We should never get here.
-        assert False, 'Internal error'
+        assert False, "Internal error"
 
     def visit_type_type(self, t: TypeType) -> ProperType:
         if isinstance(self.s, TypeType):
@@ -698,7 +868,7 @@ class TypeMeetVisitor(TypeVisitor[ProperType]):
             if not isinstance(typ, NoneType):
                 typ = TypeType.make_normalized(typ, line=t.line)
             return typ
-        elif isinstance(self.s, Instance) and self.s.type.fullname == 'builtins.type':
+        elif isinstance(self.s, Instance) and self.s.type.fullname == "builtins.type":
             return t
         elif isinstance(self.s, CallableType):
             return self.meet(t, self.s)
@@ -706,7 +876,7 @@ class TypeMeetVisitor(TypeVisitor[ProperType]):
             return self.default(self.s)
 
     def visit_type_alias_type(self, t: TypeAliasType) -> ProperType:
-        assert False, "This should be never called, got {}".format(t)
+        assert False, f"This should be never called, got {t}"
 
     def meet(self, s: Type, t: Type) -> ProperType:
         return meet_types(s, t)
@@ -724,23 +894,25 @@ class TypeMeetVisitor(TypeVisitor[ProperType]):
 def meet_similar_callables(t: CallableType, s: CallableType) -> CallableType:
     from mypy.join import join_types
 
-    arg_types: List[Type] = []
+    arg_types: list[Type] = []
     for i in range(len(t.arg_types)):
         arg_types.append(join_types(t.arg_types[i], s.arg_types[i]))
     # TODO in combine_similar_callables also applies here (names and kinds)
     # The fallback type can be either 'function' or 'type'. The result should have 'function' as
     # fallback only if both operands have it as 'function'.
-    if t.fallback.type.fullname != 'builtins.function':
+    if t.fallback.type.fullname != "builtins.function":
         fallback = t.fallback
     else:
         fallback = s.fallback
-    return t.copy_modified(arg_types=arg_types,
-                           ret_type=meet_types(t.ret_type, s.ret_type),
-                           fallback=fallback,
-                           name=None)
+    return t.copy_modified(
+        arg_types=arg_types,
+        ret_type=meet_types(t.ret_type, s.ret_type),
+        fallback=fallback,
+        name=None,
+    )
 
 
-def meet_type_list(types: List[Type]) -> Type:
+def meet_type_list(types: list[Type]) -> Type:
     if not types:
         # This should probably be builtins.object but that is hard to get and
         # it doesn't matter for any current users.
@@ -768,11 +940,12 @@ def typed_dict_mapping_pair(left: Type, right: Type) -> bool:
         _, other = right, left
     else:
         return False
-    return isinstance(other, Instance) and other.type.has_base('typing.Mapping')
+    return isinstance(other, Instance) and other.type.has_base("typing.Mapping")
 
 
-def typed_dict_mapping_overlap(left: Type, right: Type,
-                               overlapping: Callable[[Type, Type], bool]) -> bool:
+def typed_dict_mapping_overlap(
+    left: Type, right: Type, overlapping: Callable[[Type, Type], bool]
+) -> bool:
     """Check if a TypedDict type is overlapping with a Mapping.
 
     The basic logic here consists of two rules:
@@ -812,7 +985,7 @@ def typed_dict_mapping_overlap(left: Type, right: Type,
         assert isinstance(right, TypedDictType)
         typed, other = right, left
 
-    mapping = next(base for base in other.type.mro if base.fullname == 'typing.Mapping')
+    mapping = next(base for base in other.type.mro if base.fullname == "typing.Mapping")
     other = map_instance_to_supertype(other, mapping)
     key_type, value_type = get_proper_types(other.args)
 
